@@ -369,7 +369,8 @@ formattedTicksAndLabels.default <-
     {
         at <-
             if (have.log && !equispaced.log) # FIXME: num.limit instead of rng?
-                checkArgsAndCall(axisTicks, list(usr = log10(logbase^rng), log = TRUE, axp = NULL, ...))
+                checkArgsAndCall(axisTicks, list(usr = log10(logbase^rng),
+                                                 log = TRUE, axp = NULL, ...))
             else
                 checkArgsAndCall(pretty, list(x = x[is.finite(x)], ...))
     }
@@ -505,164 +506,6 @@ formattedTicksAndLabels.expression <-
 
 
 
-## prettyDate copied here temporarily.
-## remove after it appears in R 2.12 ?
-
-prettyDate_TMP <-
-    function(x, n = 5, min.n = n %/% 2, ...)
-{
-    isDate <- inherits(x, "Date")
-    x <- as.POSIXct(x)
-    if (isDate) # the timezone *does* matter
-        attr(x, "tzone") <- "GMT"
-    zz <- range(x, na.rm = TRUE)
-    if (diff(as.numeric(zz)) == 0)# one value only
-        zz <- zz + c(0,60)
-    
-    ## specify the set of pretty timesteps
-    MIN <- 60
-    HOUR <- MIN * 60
-    DAY <- HOUR * 24
-    YEAR <- DAY * 365.25
-    MONTH <- YEAR / 12
-    steps <-
-        list("1 sec" = list(1, format = "%S", start = "mins"),
-             "2 secs" = list(2),
-             "5 secs" = list(5),
-             "10 secs" = list(10),
-             "15 secs" = list(15),
-             "30 secs" = list(30, format = "%H:%M:%S"),
-             "1 min" = list(1*MIN, format = "%H:%M"),
-             "2 mins" = list(2*MIN, start = "hours"),
-             "5 mins" = list(5*MIN),
-             "10 mins" = list(10*MIN),
-             "15 mins" = list(15*MIN),
-             "30 mins" = list(30*MIN),
-             "1 hour" = list(1*HOUR),
-             "3 hours" = list(3*HOUR, start = "days"),
-             "6 hours" = list(6*HOUR, format = "%b %d %H:%M"),
-             "12 hours" = list(12*HOUR),
-             "1 DSTday" = list(1*DAY, format = "%b %d"),
-             "2 DSTdays" = list(2*DAY),
-             "1 week" = list(7*DAY, start = "weeks"),
-             "halfmonth" = list(MONTH/2, start = "months"),
-             "1 month" = list(1*MONTH, format = "%b"),
-             "3 months" = list(3*MONTH, start = "years"),
-             "6 months" = list(6*MONTH, format = "%Y-%m"),
-             "1 year" = list(1*YEAR, format = "%Y"),
-             "2 years" = list(2*YEAR, start = "decades"),
-             "5 years" = list(5*YEAR),
-             "10 years" = list(10*YEAR),
-             "20 years" = list(20*YEAR, start = "centuries"),
-             "50 years" = list(50*YEAR),
-             "100 years" = list(100*YEAR),
-             "200 years" = list(200*YEAR),
-             "500 years" = list(500*YEAR),
-             "1000 years" = list(1000*YEAR))
-    ## carry forward 'format' and 'start' to following steps
-    for (i in seq_along(steps)) {
-        if (is.null(steps[[i]]$format))
-            steps[[i]]$format <- steps[[i-1]]$format
-        if (is.null(steps[[i]]$start))
-            steps[[i]]$start <- steps[[i-1]]$start
-        steps[[i]]$spec <- names(steps)[i]
-    }
-    ## crudely work out number of steps in the given interval
-    xspan <- diff(as.numeric(zz))
-    nsteps <- sapply(steps, function(s) {
-        xspan / s[[1]]
-    })
-    init.i <- which.min(abs(nsteps - n))
-    ## calculate actual number of ticks in the given interval
-    calcSteps <- function(s) {
-        startTime <- trunc_POSIXt_TMP(min(zz), units = s$start) ## FIXME: should be trunc() eventually
-        if (identical(s$spec, "halfmonth")) {
-            at <- seq(startTime, max(zz), by = "months")
-            at2 <- as.POSIXlt(at)
-            at2$mday <- 15L
-            at <- structure(sort(c(as.POSIXct(at), as.POSIXct(at2))), 
-                            tzone = attr(at, "tzone"))
-        } else {
-            at <- seq(startTime, max(zz), by = s$spec)
-        }
-        at <- at[(min(zz) <= at) & (at <= max(zz))]
-        at
-    }
-    init.at <- calcSteps(steps[[init.i]])
-    init.n <- length(init.at) - 1L
-    ## bump it up if below acceptable threshold
-    while (init.n < min.n) {
-        init.i <- init.i - 1
-        if (init.i == 0) stop("range too small for min.n")
-        init.at <- calcSteps(steps[[init.i]])
-        init.n <- length(init.at) - 1L
-    }
-    makeOutput <- function(at, s) {
-        flabels <- format(at, s$format)
-        ans <-
-            if (isDate) as.Date(round(at, "days"))
-            else as.POSIXct(at)
-        attr(ans, "labels") <- flabels
-        ans
-    }
-    if (init.n == n) ## perfect
-        return(makeOutput(init.at, steps[[init.i]]))
-    if (init.n > n) {
-        ## too many ticks
-        new.i <- init.i + 1
-        new.i <- min(new.i, length(steps))
-    } else {
-        ## too few ticks
-        new.i <- init.i - 1
-        new.i <- max(new.i, 1)
-    }
-    new.at <- calcSteps(steps[[new.i]])
-    new.n <- length(new.at) - 1L
-    ## work out whether new.at or init.at is better
-    if (new.n < min.n)
-        new.n <- -Inf
-    if (abs(new.n - n) < abs(init.n - n))
-        return(makeOutput(new.at, steps[[new.i]]))
-    else
-        return(makeOutput(init.at, steps[[init.i]]))
-}
-
-trunc_POSIXt_TMP <-
-    function(x, units = c("secs", "mins", "hours", "days",
-                "weeks", "months", "years", "decades", "centuries"),
-             start.on.monday = TRUE)
-{
-    x <- as.POSIXlt(x)
-    if (units %in% c("secs", "mins", "hours", "days"))
-        return(base::trunc.POSIXt(x, units))
-    x <- base::trunc.POSIXt(x, "days")
-    if (length(x$sec))
-        switch(units,
-               weeks = {
-                   x$mday <- x$mday - x$wday
-                   if (start.on.monday)
-                       x$mday <- x$mday + ifelse(x$wday > 0L, 1L, -6L)
-               },
-               months = {
-                   x$mday <- 1
-               },
-               years = {
-                   x$mday <- 1
-                   x$mon <- 0
-               },
-               decades = {
-                   x$mday <- 1
-                   x$mon <- 0
-                   x$year <- (x$year %/% 10) * 10
-               },
-               centuries = {
-                   x$mday <- 1
-                   x$mon <- 0
-                   x$year <- (x$year %/% 100) * 100
-               })
-    x
-}
-
 
 ## method for "Date" (using pretty.Date)
 ## and POXIXct (using pretty.POSIXt)
@@ -695,7 +538,7 @@ formattedTicksAndLabels.POSIXct <-
     mat <- is.logical(at)
     if (mat) ## at not explicitly specified
     {
-        at <- prettyDate_TMP(x, ...) ## FIXME: should be pretty() eventually
+        at <- pretty(x, ...)
         if (is.null(format.posixt))
             labels <- attr(at, "labels")
         else
@@ -838,6 +681,7 @@ panel.axis <-
              line.alpha = axis.line$alpha)
 {
     side <- match.arg(side)
+    at.missing <- missing(at)
     orientation <- if (outside) "outer" else "inner"
     cpl <- current.panel.limits()
     scale.range <-
@@ -858,6 +702,8 @@ panel.axis <-
         labels <-
             if (labels) format(at, trim = TRUE)
             else NULL
+    else if (at.missing)
+        warning("specifying 'labels' but not 'at' may lead to unexpected results")
 
     if (check.overlap) ## remove ticks close to limits
     {
@@ -869,18 +715,6 @@ panel.axis <-
     keep.at <- at >= scale.range[1] & at <= scale.range[2]
     at <- at[keep.at]
     labels <- labels[keep.at]
-    keep.labels <- TRUE
-
-    ## could possibly treat ticks and labels separately, but that
-    ## wasn't liked much.   I'll still keep 'keep.labels' around for now,
-    ## but it's currently redundant
-
-    ## if (check.overlap)
-    ## {
-    ##     pad <- lattice.getOption("something else")
-    ##     scale.range <- extend.limits(scale.range, prop = -pad)
-    ##     keep.labels <- at >= scale.range[1] & at <= scale.range[2]
-    ## }
 
     nal <- length(at) / 2 + 0.5
     all.id <- seq_along(at)
@@ -892,6 +726,7 @@ panel.axis <-
             if (which.half == "lower") lower.id else upper.id
         }
         else rep(TRUE, length(all.id))
+    if (!any(axid)) return(invisible())
 
     gp.line <- gpar(col = line.col, alpha = line.alpha,
                     lty = line.lty, lwd = line.lwd)
@@ -976,8 +811,8 @@ panel.axis <-
         }
         switch(side,
                bottom =
-               grid.text(label = labels[axid & keep.labels],
-                         x = unit(at[axid & keep.labels], "native"),
+               grid.text(label = labels[axid],
+                         x = unit(at[axid], "native"),
                          y = orient.factor * lab.unit,
                          rot = rot[1],
                          check.overlap = check.overlap,
@@ -986,8 +821,8 @@ panel.axis <-
                                                  type="panel"),
                          gp = gp.text),
                top =
-               grid.text(label = labels[axid & keep.labels],
-                         x = unit(at[axid & keep.labels], "native"),
+               grid.text(label = labels[axid],
+                         x = unit(at[axid], "native"),
                          y = unit(1, "npc") - orient.factor * lab.unit,
                          rot = rot[1],
                          check.overlap = check.overlap,
@@ -996,8 +831,8 @@ panel.axis <-
                                                  type="panel"),
                          gp = gp.text),
                left =
-               grid.text(label = labels[axid & keep.labels],
-                         y = unit(at[axid & keep.labels], "native"),
+               grid.text(label = labels[axid],
+                         y = unit(at[axid], "native"),
                          x = orient.factor * lab.unit,
                          rot = rot[2],
                          check.overlap = check.overlap,
@@ -1006,8 +841,8 @@ panel.axis <-
                                                  type="panel"),
                          gp = gp.text),
                right =
-               grid.text(label = labels[axid & keep.labels],
-                         y = unit(at[axid & keep.labels], "native"),
+               grid.text(label = labels[axid],
+                         y = unit(at[axid], "native"),
                          x = unit(1, "npc") - orient.factor * lab.unit,
                          rot = rot[2],
                          check.overlap = check.overlap,
@@ -1019,3 +854,4 @@ panel.axis <-
     invisible()
 }
 
+
