@@ -51,7 +51,7 @@ panel.abline <-
     {
         p <- length(coefa <- as.vector(coef(a)))
         if (p > 2)
-            warning("only using the first two of ", p, "regression coefficients")
+            warning(gettextf("only using the first two of %d regression coefficients", p))
         islm <- inherits(a, "lm")
         noInt <- if (islm)
             !as.logical(attr(stats::terms(a), "intercept"))
@@ -87,39 +87,6 @@ panel.abline <-
         xx <- cpl$xlim
         yy <- cpl$ylim
 
-        if (FALSE) ## wrong when ll() == 0
-        {
-          x <- numeric(0)
-          y <- numeric(0)
-          ll <- function(i, j, k, l)
-              (yy[j] - coeff[1] - coeff[2] * xx[i]) *
-                  (yy[l] - coeff[1] - coeff[2] * xx[k])
-          if (ll(1,1,2,1) <= 0) {
-              y <- c(y, yy[1])
-              x <- c(x, (yy[1] - coeff[1]) / coeff[2])
-          }
-          if (ll(2,1,2,2) <= 0) {
-              x <- c(x, xx[2])
-              y <- c(y, coeff[1] + coeff[2] * xx[2])
-          }
-          if (ll(2,2,1,2) <= 0) {
-              y <- c(y, yy[2])
-              x <- c(x, (yy[2]-coeff[1])/coeff[2])
-          }
-          if (ll(1,2,1,1) <= 0) {
-              x <- c(x, xx[1])
-              y <- c(y, coeff[1] + coeff[2] * xx[1])
-          }
-          str(list(x = x, y = y))
-          panel.lines(x = x, y = y,
-                      col = col.line,
-                      lty = lty,
-                      lwd = lwd,
-                      alpha = alpha,
-                      ...,
-                      identifier = id)
-        }
-
         ## There are six = choose(4, 2) possibilities (plus edge cases):
         ##      I           II           III          IV            V           VI
         ## 4----C----3  4---------3  4---------3  4---------3  4---------3  4---------3
@@ -132,8 +99,9 @@ panel.abline <-
 
         sign.dist.from.line <- function(x, y)
         {
-            as.integer(sign(y - coeff[1] - coeff[2] * x)) ## / sqrt(1 + coeff[2]^2)
+            as.integer(sign(zapsmall(y - coeff[1] - coeff[2] * x)))
         }
+        force(sign.dist.from.line)
         sign.corners <-
             with(cpl,
                  sign.dist.from.line(c(xlim[1], xlim[2], xlim[2], xlim[1]),
@@ -155,14 +123,16 @@ panel.abline <-
                            ...,
                            identifier = id)
         }
+        ## Do opposite sides first to make sure diagonal edge cases
+        ## proceed along the correct branch
         if (D && B) # Case I
             drawfun(xx[1], yfun(xx[1]), xx[2], yfun(xx[2]), ...)
+        else if (A && C) # Case IV
+            drawfun(xfun(yy[1]), yy[1], xfun(yy[2]), yy[2], ...)
         else if (D && C) # Case II
             drawfun(xx[1], yfun(xx[1]), xfun(yy[2]), yy[2], ...)
         else if (A && B) # Case III
             drawfun(xfun(yy[1]), yy[1], xx[2], yfun(xx[2]), ...)
-        else if (A && C) # Case IV
-            drawfun(xfun(yy[1]), yy[1], xfun(yy[2]), yy[2], ...)
         else if (B && C) # Case  V
             drawfun(xfun(yy[1]), yy[1], xfun(yy[2]), yy[2], ...)
         else if (A && D) # Case VI
@@ -415,7 +385,7 @@ panel.grid <-
                 y <- as.character(y)
             mostattributes(scale) <- attributes(y)
         }
-        #at <- pretty(scale, n = n) ## FIXME: use pretty eventually
+        ## at <- pretty(scale, n = n)
         at <- formattedTicksAndLabels(scale, n = n)$at
         at <- at[at > min(scale) & at < max(scale)]
         grid.segments(y0 = at,
@@ -435,7 +405,7 @@ panel.grid <-
                 x <- as.character(y)
             mostattributes(scale) <- attributes(x)
         }
-        #at <- pretty(scale, n = n) ## FIXME: use pretty eventually
+        ## at <- pretty(scale, n = n)
         at <- formattedTicksAndLabels(scale, n = n)$at
         at <- at[at > min(scale) & at < max(scale)]
         grid.segments(x0 = at,
