@@ -84,7 +84,8 @@ diag.panel.splom <-
                                     at = if (is.null(at)) TRUE else at,
                                     labels = if (is.null(labels)) TRUE else labels,
                                     logsc = FALSE,
-                                    ..., n = tick.number)
+                                    ## ...,
+                                    n = tick.number)
         for (side in c("left", "top", "right", "bottom"))
             panel.axis(side = side,
                        at = axis.details$at,
@@ -133,7 +134,7 @@ panel.pairs <-
 
              ## prepanel.limits = function(x) if (is.factor(x)) levels(x) else
              ## extend.limits(range(as.numeric(x), finite = TRUE)),
-             prepanel.limits = scale.limits,
+             prepanel.limits = scale_limits,
 
              varnames = colnames(z),
              varname.col = add.text$col,
@@ -234,35 +235,36 @@ panel.pairs <-
                     ##     labels <- labels[axid]
                     ## }
 
-                    diag.panel(x = z[subscripts, j],
-                               varname = varnames[i],
-                               limits = lim[[i]],
-                               at = if (is.list(pscales)) pscales[[i]]$at else NULL, 
-                               labels = if (is.list(pscales)) pscales[[i]]$lab else NULL,
-                               draw = draw,
-                               tick.number = if (is.numeric(pscales)) pscales else 5,
+                    if (is.function(diag.panel)) # else ignore
+                        diag.panel(x = z[subscripts, j],
+                                   varname = varnames[i],
+                                   limits = lim[[i]],
+                                   at = if (is.list(pscales)) pscales[[i]]$at else NULL, 
+                                   labels = if (is.list(pscales)) pscales[[i]]$lab else NULL,
+                                   draw = draw,
+                                   tick.number = if (is.numeric(pscales)) pscales else 5,
 
-                               varname.col = varname.col,
-                               varname.cex = varname.cex,
-                               varname.font = varname.font,
-                               varname.fontfamily = varname.fontfamily,
-                               varname.fontface = varname.fontface,
+                                   varname.col = varname.col,
+                                   varname.cex = varname.cex,
+                                   varname.font = varname.font,
+                                   varname.fontfamily = varname.fontfamily,
+                                   varname.fontface = varname.fontface,
 
-                               axis.text.col = axis.text.col,
-                               axis.text.cex = axis.text.cex,
-                               axis.text.font = axis.text.font,
-                               axis.text.fontfamily = axis.text.fontfamily,
-                               axis.text.fontface = axis.text.fontface,
-                               axis.text.lineheight = axis.text.lineheight,
+                                   axis.text.col = axis.text.col,
+                                   axis.text.cex = axis.text.cex,
+                                   axis.text.font = axis.text.font,
+                                   axis.text.fontfamily = axis.text.fontfamily,
+                                   axis.text.fontface = axis.text.fontface,
+                                   axis.text.lineheight = axis.text.lineheight,
 
-                               axis.line.col = axis.line.col,
-                               axis.line.lty = axis.line.lty,
-                               axis.line.lwd = axis.line.lwd,
-                               axis.line.alpha = axis.line.alpha,
-                               axis.line.tck = axis.line.tck,
+                                   axis.line.col = axis.line.col,
+                                   axis.line.lty = axis.line.lty,
+                                   axis.line.lwd = axis.line.lwd,
+                                   axis.line.alpha = axis.line.alpha,
+                                   axis.line.tck = axis.line.tck,
 
-                               i = i, j= j,
-                               ...)
+                                   i = i, j= j,
+                                   ...)
 
                     grid.rect(name = trellis.grobname("pairs.border",
                                 type="panel"),
@@ -328,6 +330,7 @@ splom.matrix <-
 splom.data.frame <-
     function(x, data = NULL, ..., groups = NULL, subset = TRUE)
 {
+    ocall <- sys.call(); ocall[[1]] <- quote(splom)
     ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
@@ -338,7 +341,7 @@ splom.data.frame <-
     ## ccall$groups <- groups
     ## ccall$subset <- subset
     ccall[[1]] <- quote(lattice::splom)
-    eval.parent(ccall)
+    modifyList(eval.parent(ccall), list(call = ocall))
 }
 
 
@@ -346,7 +349,7 @@ splom.data.frame <-
 splom.formula <-
     function(x,
              data = NULL,
-             auto.key = FALSE,
+             auto.key = lattice.getOption("default.args")$auto.key,
              aspect = 1,
              between = list(x = 0.5, y = 0.5),
              panel = lattice.getOption("panel.splom"),
@@ -439,7 +442,7 @@ splom.formula <-
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
-    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(splom)
+    foo$call <- sys.call(); foo$call[[1]] <- quote(splom)
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
@@ -535,27 +538,29 @@ splom.formula <-
 
     if (is.null(foo$legend) && needAutoKey(auto.key, groups))
     {
+        # provide smart defaults for auto key based on 'type'
+        type <- dots$type
+        if (is.character(type) && length(type) > 0)
+        {
+            points <- any(type %in% "p")
+            lines <- any(type %in% c("l", "b", "o", "h", "s", "S", "a",
+                                     "smooth", "spline", "r"))
+            keytype <- if (any(type %in% c("b", "o"))) "o" else "l"
+        }
+        else
+        {
+            points <- TRUE
+            lines <- FALSE
+            keytype <- "l"
+        }
         foo$legend <-
-            list(list(fun = "drawSimpleKey",
-                      args =
-                      updateList(list(text = levels(as.factor(groups)),
-                                      points = TRUE,
-                                      rectangles = FALSE,
-                                      lines = FALSE), 
-                                 if (is.list(auto.key)) auto.key else list())))
-        foo$legend[[1]]$x <- foo$legend[[1]]$args$x
-        foo$legend[[1]]$y <- foo$legend[[1]]$args$y
-        foo$legend[[1]]$corner <- foo$legend[[1]]$args$corner
-
-        names(foo$legend) <- 
-            if (any(c("x", "y", "corner") %in% names(foo$legend[[1]]$args)))
-                "inside"
-            else
-                "top"
-        if (!is.null(foo$legend[[1]]$args$space))
-            names(foo$legend) <- foo$legend[[1]]$args$space
+            autoKeyLegend(list(text = levels(as.factor(groups)),
+                               points = points,
+                               rectangles = FALSE,
+                               lines = lines,
+                               type = keytype),
+                          auto.key)
     }
-
     class(foo) <- "trellis"
     foo
 }
